Exception: Commands::UsageError
- Inherits:
-
ArgumentError
- Object
- ArgumentError
- Commands::UsageError
- Defined in:
- lib/droxi/commands.rb
Overview
Exception indicating that a client command was given the wrong number of arguments.
Exception indicating that a client command was given the wrong number of arguments.